@@ -21,7 +21,11 @@
 */
 
 #include <config.h>
-#include <libgnome/gnome-i18n.h>
+#ifdef GNOME_GTKHTML_EDITOR_SHLIB
+#include <glib/gi18n-lib.h>
+#else
+#include <glib/gi18n.h>
+#endif
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include "replace.h"
@@ -31,6 +35,7 @@
 struct _GtkHTMLReplaceAskDialog {
 	GtkDialog  *dialog;
 	HTMLEngine *engine;
+	gboolean finished;
 };
 
 typedef struct _GtkHTMLReplaceAskDialog GtkHTMLReplaceAskDialog;
@@ -49,21 +54,21 @@ struct _GtkHTMLReplaceDialog {
 static void
 ask_dialog_response (GtkDialog *dialog, gint response_id, GtkHTMLReplaceAskDialog *d)
 {
+	HTMLReplaceQueryAnswer answer = RQA_Cancel;
+
 	switch (response_id) {
-	case GTK_RESPONSE_DELETE_EVENT:
-	case GTK_RESPONSE_CLOSE:
-		html_engine_replace_do (d->engine, RQA_Cancel);
-		break;
 	case 0: /* Replace */
-		html_engine_replace_do (d->engine, RQA_Replace);
+		answer = RQA_Replace;
 		break;
 	case 1: /* Replace All */
-		html_engine_replace_do (d->engine, RQA_ReplaceAll);
+		answer = RQA_ReplaceAll;
 		break;
 	case 2: /* Next */
-		html_engine_replace_do (d->engine, RQA_Next);
+		answer = RQA_Next;
 		break;
 	}
+
+	d->finished = html_engine_replace_do (d->engine, answer);
 }
 
 static GtkHTMLReplaceAskDialog *
@@ -72,13 +77,15 @@ ask_dialog_new (HTMLEngine *e)
 	GtkHTMLReplaceAskDialog *d;
 
 	d = g_new (GtkHTMLReplaceAskDialog, 1);
+	/* we use CANCEL response for close, because we want Esc to close the dialog - see gtkdialog.c */
 	d->dialog = GTK_DIALOG (gtk_dialog_new_with_buttons (_("Replace confirmation"), NULL, 0,
-							     _("Replace all"), 1,
-							     _("Next"), 2,
-							     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-							     _("Replace"), 0,
+							     _("Replace _All"), 1,
+							     _("_Next"), 2,
+							     GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
+							     _("_Replace"), 0,
 							     NULL));
 	d->engine = e;
+	d->finished = FALSE;
 
 	gnome_window_icon_set_from_file (GTK_WINDOW (d->dialog), ICONDIR "/search-and-replace-24.png");
 	g_signal_connect (d->dialog, "response", G_CALLBACK (ask_dialog_response), d);
@@ -92,8 +99,10 @@ ask (HTMLEngine *e, gpointer data)
 	GtkHTMLReplaceAskDialog *ask_dialog;
 
 	ask_dialog = ask_dialog_new (e);
-	gtk_dialog_run (ask_dialog->dialog);
+	while (!ask_dialog->finished)
+		gtk_dialog_run (ask_dialog->dialog);
 	gtk_widget_destroy (GTK_WIDGET (ask_dialog->dialog));
+	gtk_widget_grab_focus (GTK_WIDGET (ask_dialog->engine->widget));
 	g_free (ask_dialog);
 }
 
@@ -132,6 +141,10 @@ replace_dialog_response (GtkDialog *dialog, gint response_id, GtkHTMLReplaceDial
 				     GTK_TOGGLE_BUTTON (d->backward)->active == 0, FALSE,
 				     ask, d);
 		break;
+	case GTK_RESPONSE_CANCEL:
+	case GTK_RESPONSE_CLOSE:
+		gtk_widget_grab_focus (GTK_WIDGET (d->html));
+		break;
 	}
 }
 
@@ -150,8 +163,9 @@ gtk_html_replace_dialog_new (GtkHTML *html, GtkHTMLControlData *cd)
 	GtkWidget *table;
 	GtkWidget *label;
 
+	/* we use CANCEL response for close, because we want Esc to close the dialog - see gtkdialog.c */
 	dialog->dialog         = GTK_DIALOG (gtk_dialog_new_with_buttons (_("Replace"), NULL, 0,
-									  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+									  GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 									  GTK_STOCK_FIND_AND_REPLACE, 0,
 									  NULL));
 
@@ -169,10 +183,12 @@ gtk_html_replace_dialog_new (GtkHTML *html, GtkHTMLControlData *cd)
 		gtk_entry_set_text (GTK_ENTRY (dialog->entry_replace), cd->replace_text_replace);
 
 	gtk_table_set_col_spacings (GTK_TABLE (table), 3);
-	label = gtk_label_new (_("Replace:"));
+	label = gtk_label_new_with_mnemonic (_("R_eplace:"));
+	gtk_label_set_mnemonic_widget (label,dialog-> entry_search);
 	gtk_misc_set_alignment (GTK_MISC (label), .0, .5);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
-	label = gtk_label_new (_("With:"));
+	label = gtk_label_new_with_mnemonic (_("_With:"));
+	gtk_label_set_mnemonic_widget (label, dialog->entry_replace);
 	gtk_misc_set_alignment (GTK_MISC (label), .0, .5);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 
